package Principal;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Stack;

import Clases.*;
import Gestores.GestorLibretaDirecciones;
import Gestores.GestorNuevoMensaje;
import Gestores.GestorPaginaPrincipal;

public class Main {

	public static String[] crearPulsacion(String tecla){
		return new String[]{"xte", "-x", ":0", "key "+tecla};
	}
	
	public static String[] crearCombinacion(String seq){
		return crearCombinacion (seq.replaceAll(" ","").split(","));
	}
	public static String[] crearCombinacion(String[] teclas){
		int i = 0;
		ArrayList<String> res = new ArrayList<String>();
		res.add("xte"); res.add("-x"); res.add(":0");
		
		for (i = 0; i < teclas.length; i++){
			res.add("keydown "+teclas[i]);
		}
		
		for (i = teclas.length-1; i >= 0; i--){
			res.add("keyup "+teclas[i]);
		}
		
		String[] accion = new String[res.size()];
		accion = res.toArray(accion);
		return accion;
	}
		
	public static String[] crearSecuencia(String seq){
		return crearSecuencia (seq.replaceAll(" ","").split(","));
	}
	public static String[] crearSecuencia(String[] teclas){
		if (teclas.length == 0) return null;
		int i = 0;
		ArrayList<String> res = new ArrayList<String>();
		res.add("xte"); res.add("-x"); res.add(":0");
		
		for (i = 0; i < teclas.length-1; i++){
			res.add("key "+teclas[i]);
			res.add("usleep 500");
		}
		res.add("key "+teclas[teclas.length-1]);
		
		String[] accion = new String[res.size()];
		accion = res.toArray(accion);
		return accion;
	}
	

	
	public static Automata crearAutomataOriginal(){
		Automata aut = new Automata(new Estado("paginaPrincipal"));
		
		// Pagina Principal
		GestorPaginaPrincipal gpp = new GestorPaginaPrincipal();
		gpp.rellenarPaginaPrincipal(aut);
		
		// Nuevo Mensaje
		GestorNuevoMensaje gnm = new GestorNuevoMensaje();
		gnm.rellenarNuevoMensaje(aut);
			
		// Libreta de Direcciones
		GestorLibretaDirecciones gld = new GestorLibretaDirecciones();
		gld.rellenarLibretaDirecciones(aut);
		return aut;
	}
	
	

	public static void printToLog(String message, String filename) throws IOException{
			BufferedWriter out = new BufferedWriter(new FileWriter(filename,true));
			out.write(message);	
			out.close();
		
	}
	
	// Limpia los logs manteniendo un backup del último, para imprimir en ellos desde cero.
	static void deleteLogs(String filename) throws IOException{
		Runtime.getRuntime().exec("mv "+filename+" "+filename+".old");
		Runtime.getRuntime().exec("rm -rf "+filename);
	}
	
	@SuppressWarnings("static-access")
	public static void main(String[] args) throws IOException, InterruptedException {
		deleteLogs("log_interprete");
		File f = new File("/tmp/state");
		f.delete();		
		f = new File("salida_reconocedor");
		f.delete();
		
		int iteration = 1;
		Stack<String> ventanas = new Stack<String>();

		// Creamos el autómata
		Automata aut = crearAutomataOriginal();
		System.out.println("Automata creado.\n Empezaremos en el estado "+
					aut.getEstadoActual().getNombre()+"("+aut.getEstadoInicial().getIndice()+")\n");
		
		ventanas.push(aut.getEstadoInicial().getNombre());
		// Bucle principal
		while(true){
			System.out.println("It. "+iteration+
					" Estado: "+aut.getEstadoActual().getNombre()+
					"("+aut.getEstadoActual().getIndice()+")");
			
			// Insertamos el índice del estado actual en /tmp/state, para que el reconocedor lo capte
			BufferedWriter out = new BufferedWriter(new FileWriter("/tmp/state",true));
			out.write(String.valueOf(iteration)+"\n");
			out.write(String.valueOf(aut.getEstadoActual().getIndice()));	
			out.close();			
			
			boolean flag=false;
			
			System.out.println("Esperando al reconocedor...");
			
			BufferedReader br = null;
			while (!flag){
				// Comprobamos para ver cuándo podemos leer el fichero...
				while(!f.exists()){Thread.currentThread().sleep(100); }
				
				// Borramos el fichero del estado, para evitar problemas
				File f2 = new File("/tmp/state");
				f2.delete();
				
				// Leemos la línea
				FileInputStream in = new FileInputStream(f);
				br = new BufferedReader(new InputStreamReader(in));
				String line_1 = br.readLine();
				if (Integer.valueOf(line_1)==iteration) flag = true;
			}
			String line = br.readLine();
			br.close();
			System.out.println("Línea capturada: \""+line+"\"\n");
			
			// Formamos el principio de la línea del log
			String logString = "It "+String.valueOf(iteration)+": "+aut.getEstadoActual().getNombre()+
					"("+aut.getEstadoActual().getIndice()+") ---["+line+"]---> ";
			
			// Eliminamos el fichero, para que se pueda utilizar para la siguiente orden
			f.delete();
			
			System.out.print(aut.getEstadoActual().getNombre()+
					"("+aut.getEstadoActual().getIndice()+") ----[\""+line+"\" ");
			// A continuación, a partir del estado actual del autómata y de la línea...
			// ... hacemos la transición al nuevo estado, emitiendo la acción necesaria.
			aut.trans(line);
			
			// Acabamos la línea del log y la imprimimos
			logString = logString+aut.getEstadoActual().getNombre()+
					"("+aut.getEstadoActual().getIndice()+")\n";
			printToLog(logString,"log_interprete");
			
			System.out.println("]---->\n"+aut.getEstadoActual().getNombre()+
					"("+aut.getEstadoActual().getIndice()+")\n---------------------------");
			
			if (aut.getEstadoActual().getNombre().equalsIgnoreCase("estadoFin")){
				BufferedWriter out2 = new BufferedWriter(new FileWriter("/tmp/state",true));
				out2.write("0\n");
				out2.write(String.valueOf(aut.getEstadoActual().getIndice()));	
				out2.close();
				System.exit(0);
			}
					
			// Incrementamos el contador de iteraciones, utilizado para el control en ficheros
			iteration++;
		}
		
			
		
	}

}
